<?php
/**
 * AsalaeCore\Error\ErrorTrap
 */

namespace AsalaeCore\Error;

use AsalaeCore\Error\Renderer\ConsoleErrorRenderer;
use AsalaeCore\Error\Renderer\HtmlErrorRenderer;
use Cake\Error\ErrorTrap as CakeErrorTrap;
use Cake\Error\PhpError;
use Cake\Log\Log;

/**
 * Gestions d'erreurs de l'application
 *
 * @category    Error
 *
 * @author      Libriciel SCOP <contact@libriciel.coop>
 * @copyright   (c) 2022, Libriciel
 * @license     https://www.gnu.org/licenses/agpl-3.0.txt
 */
class ErrorTrap extends CakeErrorTrap
{
    /**
     * Configuration options. Generally these are defined in config/app.php
     *
     * - `errorLevel` - int - The level of errors you are interested in capturing.
     * - `errorRenderer` - string - The class name of render errors with. Defaults
     *   to choosing between Html and Console based on the SAPI.
     * - `log` - boolean - Whether or not you want errors logged.
     * - `logger` - string - The class name of the error logger to use.
     * - `trace` - boolean - Whether or not backtraces should be included in
     *   logged errors.
     *
     * @var array<string, mixed>
     */
    protected $_defaultConfig = [
        'errorLevel' => E_ALL,
        'errorRenderer' => null,
        'log' => true,
        'logger' => ErrorLogger::class,
        'trace' => false,
    ];

    /**
     * Choose an error renderer based on config or the SAPI
     *
     * @return class-string<ConsoleErrorRenderer|HtmlErrorRenderer>
     */
    protected function chooseErrorRenderer(): string
    {
        $config = $this->getConfig('errorRenderer');
        if ($config !== null) {
            return $config;
        }

        /** @var class-string<ConsoleErrorRenderer|HtmlErrorRenderer> */
        return PHP_SAPI === 'cli' ? ConsoleErrorRenderer::class : HtmlErrorRenderer::class;
    }

    /**
     * Logging helper method.
     *
     * @param PhpError $error The error object to log.
     * @return void
     */
    protected function logError(PhpError $error): void
    {
        $stdout = Log::getConfig('stdout');
        $stderr = Log::getConfig('stderr');
        Log::drop('stdout');
        Log::drop('stderr');
        parent::logError($error);
        if (PHP_SAPI === 'cli' && $stdout && $stderr) {
            Log::setConfig('stdout', $stdout);
            Log::setConfig('stderr', $stderr);
        }
    }
}
